class Shortcuts:
    """
    @todo move this out of the repo
    List of mapped i3 shortcuts, in gnome the syntax is different.
    """

    CTRL_W = "Control+w"
    CTRL_V = "Control+v"
    CTRL_SPACE = "Control+space"
    SUPER_SPACE = "Mod4+space"
    SUPER_A = "Mod4+A"
    SUPER_W = "Mod4+w"
    SUPER_D = "Mod4+d"
    SUPER_P = "Mod4+p"

    ALT_SPACE = "Mod1+space"
    ALT_SHIFT_SPACE = "Mod1+Shift+space"
    ALT_SHIFT_D = "Mod1+Shift+d"
    ALT_SHIFT_S = "Mod1+Shift+s"
    ALT_SHIFT_J = "Mod1+Shift+j"
    ALT_SHIFT_L = "Mod1+Shift+l"
    SUPER_H = "Mod4+h"
    SUPER_SHIFT_H = "Mod4+Shift+h"
    CTRL_SHIFT_H = "Control+Shift+h"
    CTRL_H = "Control+h"
    ALT_H = "Mod1+h"
    ALT_A = "Mod1+a"
    ALT_V = "Mod1+v"
    ALT_SHIFT_A = "Mod1+Shift+a"
    ALT_SHIFT_H = "Mod1+Shift+h"
    CTRL_B = "Control+b"
    CTRL_E = "Control+e"
    CTRL_P = "Control+p"
    CTRL_I = "Control+i"
    CTRL_K = "Control+k"
    CTRL_D = "Control+d"
    CTRL_L = "Control+l"
    CTRL_SHIFT_D = "Control+Shift+d"
    CTRL_SHIFT_K = "Control+Shift+k"
    CTRL_SHIFT_E = "Control+Shift+e"
    ALT_SHIFT_X = "Mod1+Shift+x"
    CTRL_5 = "Control+5"
    CTRL_Q = "Control+q"
    SUPER_T = "Mod4+t"
    SUPER_0 = "Mod4+0"
    SUPER_9 = "Mod4+9"
    SUPER_8 = "Mod4+8"
    SUPER_7 = "Mod4+7"

    SUPER_SHIFT_W = "Mod4+Shift+w"
    SUPER_SHIFT_V = "Mod4+Shift+v"

    CTRL_SHIFT_N = "Control+Shift+n"
    CTRL_SHIFT_9 = "Control+Shift+9"
    CTRL_SHIFT_X = "Control+Shift+x"
    CTRL_SHIFT_F = "Control+Shift+f"
    CTRL_SHIFT_8 = "Control+Shift+8"
    CTRL_SHIFT_7 = "Control+Shift+7"
    CTRL_SHIFT_6 = "Control+Shift+6"
    CTRL_SHIFT_5 = "Control+Shift+5"
    CTRL_SHIFT_4 = "Control+Shift+4"
    CTRL_SHIFT_3 = "Control+Shift+3"
    CTRL_SHIFT_1 = "Control+Shift+1"

    CTRL_SHIFT_G = "Control+Shift+g"
    CTRL_SHIFT_P = "Control+Shift+p"
    CTRL_SHIFT_W = "Control+Shift+w"
    CTRL_ALT_T = "Control+Mod1+t"
    ALT_SHIFT_T = "Mod1+Shift+t"

    ALT_SHIFT_I = "Mod1+Shift+i"
    ALT_SHIFT_0 = "Mod1+Shift+0"
    CTRL_SHIFT_0 = "Control+Shift+0"
    ALT_SHIFT_Z = "Mod1+Shift+z"
    ALT_K = "Mod1+k"
    ALT_O = "Mod1+o"
    ALT_T = "Mod1+t"
    ALT_SHIFT_K = "Mod1+Shift+k"
    ALT_D = "Mod1+d"
    ALT_I = "Mod1+i"
    ALT_P = "Mod1+p"
    ALT_SHIFT_P = "Mod1+Shift+p"
    ALT_W = "Mod1+w"
    ALT_SHIFT_W = "Mod1+Shift+w"
    ALT_U = "Mod1+u"
    ALT_N = "Mod1+n"
    ALT_M = "Mod1+m"
    SUPER_M = "Mod4+m"
    ALT_SHIFT_M = "Mod1+Shift+m"
    ALT_F = "Mod1+f"
    ALT_SHIFT_F = "Mod1+Shift+f"
    ALT_E = "Mod1+e"
    ALT_C = "Mod1+c"
    ALT_SHIFT_J = "Mod1+Shift+j"
    CTRL_J = "Control+j"
    CTRL_SHIFT_J = "Control+Shift+j"
    ALT_SHIFT_E = "Mod1+Shift+e"
    ALT_SHIFT_N = "Mod1+Shift+n"

    ALT_J = "Mod1+j"
    ALT_G = "Mod1+g"
    ALT_B = "Mod1+b"
    ALT_SHIFT_B = "Mod1+Shift+b"
    SUPER_B = "Mod4+b"
    SUPER_SHIFT_B = "Mod4+Shift+b"
    SUPER_SHIFT_1 = "Mod4+Shift+1"
    SUPER_SHIFT_2 = "Mod4+Shift+2"
    SUPER_SHIFT_3 = "Mod4+Shift+3"
    SUPER_SHIFT_4 = "Mod4+Shift+4"
    ALT_L = "Mod1+l"
    ALT_SHIFT_L = "Mod1+Shift+l"
    ALT_SHIFT_G = "Mod1+Shift+g"

    CTRL_1 = "Control+1"
    CTRL_2 = "Control+2"
    CTRL_3 = "Control+3"
    CTRL_4 = "Control+4"
    CTRL_5 = "Control+5"
    CTRL_6 = "Control+6"
    CTRL_7 = "Control+7"
    CTRL_8 = "Control+8"
    CTRL_9 = "Control+9"
    CTRL_0 = "Control+0"

    ALT_1 = "Mod1+1"
    ALT_2 = "Mod1+2"
    ALT_3 = "Mod1+3"
    ALT_4 = "Mod1+4"
    ALT_5 = "Mod1+5"
    ALT_6 = "Mod1+6"
    ALT_7 = "Mod1+7"
    ALT_8 = "Mod1+8"
    ALT_9 = "Mod1+9"
    ALT_0 = "Mod1+0"

    ALT_SHIFT_1 = "Mod1+Shift+1"
    ALT_SHIFT_2 = "Mod1+Shift+2"
    ALT_SHIFT_3 = "Mod1+Shift+3"
    ALT_SHIFT_4 = "Mod1+Shift+4"
    ALT_SHIFT_5 = "Mod1+Shift+5"
    ALT_SHIFT_6 = "Mod1+Shift+6"
    ALT_SHIFT_7 = "Mod1+Shift+7"
    ALT_SHIFT_8 = "Mod1+Shift+8"
    ALT_SHIFT_9 = "Mod1+Shift+9"
    ALT_SHIFT_0 = "Mod1+Shift+0"

    CTRL_1 = "Control+1"
    CTRL_2 = "Control+2"
    CTRL_3 = "Control+3"
    CTRL_4 = "Control+4"
    CTRL_5 = "Control+5"
    CTRL_6 = "Control+6"
    CTRL_7 = "Control+7"
    CTRL_8 = "Control+8"
    CTRL_9 = "Control+9"
    CTRL_0 = "Control+0"

    CTRL_SHIFT_1 = "Control+Shift+1"
    CTRL_SHIFT_2 = "Control+Shift+2"
    CTRL_SHIFT_3 = "Control+Shift+3"
    CTRL_SHIFT_4 = "Control+Shift+4"
    CTRL_SHIFT_5 = "Control+Shift+5"
    CTRL_SHIFT_6 = "Control+Shift+6"
    CTRL_SHIFT_7 = "Control+Shift+7"
    CTRL_SHIFT_8 = "Control+Shift+8"
    CTRL_SHIFT_9 = "Control+Shift+9"
    CTRL_SHIFT_0 = "Control+Shift+0"

    SUPER_U = "Mod4+u"
    SUPER_E = "Mod4+e"
    CAPS_E = "Mod3+e"
    SUPER_SHIFT_E = "Mod4+Shift+e"
    SUPER_SHIFT_C = "Mod4+Shift+c"
    SUPER_SHIFT_T = "Mod4+Shift+t"
    SUPER_G = "Mod4+g"
    SUPER_SHIFT_G = "Mod4+Shift+g"
    SUPER_Y = "Mod4+y"
    SUPER_I = "Mod4+i"
    SUPER_SHIFT_I = "Mod4+Shift+i"
    SUPER_SHIFT_O = "Mod4+Shift+o"
    SUPER_S = "Mod4+s"
    SUPER_C = "Mod4+c"

    SUPER_R = "Mod4+r"
    SUPER_SHIFT_R = "Mod4+Shift+r"

    ALT_R = "Mod1+r"
    ALT_SHIFT_R = "Mod1+Shift+r"
    ALT_SHIFT_V = "Mod1+Shift+v"

    @staticmethod
    @property
    def SUPER_L():
        raise Exception("Key mapped by system, should not be used")

    @staticmethod
    @property
    def SUPER_O():
        raise Exception("Key mapped by system, should not be used")

    @staticmethod
    @property
    def SUPER_V():
        raise Exception("Key mapped by system, should not be used")
